@@ -305,7 +305,8 @@ async fn register_agent(session: &Session, request_default: bool, set_trust: boo
 #[clap(
     name = "gattcat",
     about = "Swiss army knife for Bluetooth LE GATT services.",
-    author = "Sebastian Urban <surban@surban.net>"
+    author = "Sebastian Urban <surban@surban.net>",
+    version = env!("CARGO_PKG_VERSION"),
 )]
 struct Opts {
     #[clap(subcommand)]
@@ -508,7 +509,7 @@ impl DiscoverOpts {
             }
 
             let dev = adapter.device(addr)?;
-            if self.public_only && dev.address_type().await.unwrap_or_default() == AddressType::Random {
+            if self.public_only && dev.address_type().await.unwrap_or_default() == AddressType::LeRandom {
                 continue;
             }
             if let Ok(Some(_)) = dev.rssi().await {
@@ -551,7 +552,16 @@ impl DiscoverOpts {
         print_if_some(2, "TX power", dev.tx_power().await?, "dBm");
         print_if_some(2, "Paired", Some(if dev.is_paired().await? { "yes" } else { "no" }), "");
         print_if_some(2, "Trusted", Some(if dev.is_trusted().await? { "yes" } else { "no" }), "");
-        print_list(2, "Services", &dev.uuids().await?.unwrap_or_default());
+
+        print_list(
+            2,
+            "Services",
+            dev.uuids().await?.unwrap_or_default().into_iter().map(|uuid| match id::Service::try_from(uuid) {
+                Ok(name) => format!("{} ({})", name, UuidOrShort(uuid)),
+                Err(_) => format!("{}", UuidOrShort(uuid)),
+            }),
+        );
+
         for (uuid, data) in dev.service_data().await?.unwrap_or_default() {
             let lines = iter::once(String::new()).chain(to_hex(&data));
             let id = match id::Service::try_from(uuid) {
@@ -560,6 +570,7 @@ impl DiscoverOpts {
             };
             print_list(2, &format!("Service data {}", id), lines);
         }
+
         for (id, data) in dev.manufacturer_data().await?.unwrap_or_default() {
             let lines = iter::once(String::new()).chain(to_hex(&data));
             let id = match id::Manufacturer::try_from(id) {
@@ -568,6 +579,7 @@ impl DiscoverOpts {
             };
             print_list(2, &format!("Manufacturer data from {}", id), lines);
         }
+
         Ok(())
     }
 
@@ -1016,7 +1028,12 @@ impl WriteOpts {
 
         char.write_ext(
             &value,
-            &remote::CharacteristicWriteRequest { offset: 0, op_type, prepare_authorize: self.prepare_authorize },
+            &remote::CharacteristicWriteRequest {
+                offset: 0,
+                op_type,
+                prepare_authorize: self.prepare_authorize,
+                ..Default::default()
+            },
         )
         .await?;
 
@@ -1440,6 +1457,7 @@ async fn make_app(
                 ],
                 ..Default::default()
             }],
+            ..Default::default()
         };
         let control = stream::select(control_tx, control_rx).boxed();
         (adapter.serve_gatt_application(app).await?, control)
@@ -1466,6 +1484,7 @@ async fn make_app(
                 }],
                 ..Default::default()
             }],
+            ..Default::default()
         };
         (adapter.serve_gatt_application(app).await?, control.boxed())
     };
